import turtle
import random


# Quanta energia gasta cada movimento do jogador.
ENERGIA_MOVIMENTO = 20

# Quanta energia se ganha com uma captura.
ENERGIA_CAPTURA = 100

# Quanto maior mais dificil a captura.
POSSIBILIDADE_DE_FUGA = 2

# Tamanho das casas da grelha.
TAMANHO_CASAS = 100


# Preparar a janela do jogo.
turtle.Screen().setup(width=700, height=700, startx=700)
turtle.bgcolor("orange")


# Desenhar a grelha.
t = turtle.Turtle()
t.hideturtle()
t.speed(0)
t.color("dark orange")
t.width(15)

def linha_horizontal(y):
    t.up()
    t.goto(-3 * TAMANHO_CASAS, y)
    t.down()
    t.goto(3 * TAMANHO_CASAS, y)

def linha_vertical(x):
    t.up()
    t.goto(x, -3 * TAMANHO_CASAS)
    t.down()
    t.goto(x, 3 * TAMANHO_CASAS)

# Linhas horizontais da grelha
linha_horizontal(3 * TAMANHO_CASAS)
linha_horizontal(2 * TAMANHO_CASAS)
linha_horizontal(1 * TAMANHO_CASAS)
linha_horizontal(0)
linha_horizontal(-1 * TAMANHO_CASAS)
linha_horizontal(-2 * TAMANHO_CASAS)
linha_horizontal(-3 * TAMANHO_CASAS)

# Linhas verticais da grelha
linha_vertical(3 * TAMANHO_CASAS)
linha_vertical(2 * TAMANHO_CASAS)
linha_vertical(1 * TAMANHO_CASAS)
linha_vertical(0)
linha_vertical(-1 * TAMANHO_CASAS)
linha_vertical(-2 * TAMANHO_CASAS)
linha_vertical(-3 * TAMANHO_CASAS)


# Criar o fugitivo com a sua imagem.
fugitivo = turtle.Turtle()
fugitivo.up()
turtle.register_shape("boneco-c-animal.gif")
fugitivo.shape("boneco-c-animal.gif")


# Criar o jogador com a sua imagem.
jogador = turtle.Turtle()
jogador.color("white")
jogador.up()
turtle.register_shape("boneco-c-cavaleiro.gif")
jogador.shape("boneco-c-cavaleiro.gif")


# Criar a energia do jogador.
energia = turtle.Turtle()
energia.up()
energia.shape("circle")
energia.color("gold")
energia.goto(0, 3 * TAMANHO_CASAS + 30)


# Quando executada, o fugitivo move-se ao calhas.
def fugitivo_foge():
    x = random.randint(-3, 3) * TAMANHO_CASAS
    y = random.randint(-3, 3) * TAMANHO_CASAS
    fugitivo.goto(x, y)

def tenta_capturar(x, y):
    jogador.goto(x, y)
    actualiza_energia(-ENERGIA_MOVIMENTO)
    # Captura com coordenadas iguais para o jogador e fugitivo.
    if jogador.xcor() == fugitivo.xcor() and jogador.ycor() == fugitivo.ycor():
        actualiza_energia(ENERGIA_CAPTURA)
        fugitivo.circle(20)
        fugitivo_foge()
        return
    # Movimentos de fuga ao calhas.
    if random.randint(1, 6) < POSSIBILIDADE_DE_FUGA:
        fugitivo_foge()

def actualiza_energia(quanto):
    # Move a energia.
    energia.forward(quanto)
    # No limite esquerdo, derrota.
    if energia.xcor() < -3 * TAMANHO_CASAS:
        jogador.circle(20)
        fim_do_jogo("Derrota...")
    # No limite direito, ganha.
    if energia.xcor() > 3 * TAMANHO_CASAS:
        fim_do_jogo("Ganhaste!")

def fim_do_jogo(mensagem):
    # Escreve mensagem junto ao jogador e as teclas deixam de fazer efeito.
    jogador.write(mensagem, align="center", font=("Arial", 50, "bold"))
    turtle.onkey(None, "Up")
    turtle.onkey(None, "Down")
    turtle.onkey(None, "Left")
    turtle.onkey(None, "Right")

# Movimentos do jogador.
def para_cima():
    x = jogador.xcor()
    y = jogador.ycor() + TAMANHO_CASAS
    tenta_capturar(x, y)

def para_baixo():
    x = jogador.xcor()
    y = jogador.ycor() - TAMANHO_CASAS
    tenta_capturar(x, y)

def para_esquerda():
    x = jogador.xcor() - TAMANHO_CASAS
    y = jogador.ycor()
    tenta_capturar(x, y)

def para_direita():
    x = jogador.xcor() + TAMANHO_CASAS
    y = jogador.ycor()
    tenta_capturar(x, y)

# Prepara o programa para usar o teclado.
turtle.listen()

# Cada tecla executa um movimento.
turtle.onkey(para_cima, "Up")
turtle.onkey(para_baixo, "Down")
turtle.onkey(para_esquerda, "Left")
turtle.onkey(para_direita, "Right")

# Antes de iniciar, o fugitivo foge.
fugitivo_foge()

# Aguarda pelo teclado ou que se feche a janela.
turtle.mainloop()
